
package de.codedo.conflicteditor.gui;

import de.codedo.conflicteditor.ConflictsView;
import de.codedo.conflicteditor.CouchDb;
import de.codedo.conflicteditor.Difference;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.ListModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableModel;

public class ConflictEditor
{
    private ConflictEditorFrame _frame;

    public static void main(String[] args)
    {
        try
        {
            new ConflictEditor().run();
        }
        catch (Exception ex)
        {
            System.out.println("Uncaught toplevel Exception");
            ex.printStackTrace(System.out);
        }
    }

    private void run()
    {
        _frame = new ConflictEditorFrame();
        connectListeners(_frame);
        _frame.pack();
        _frame.setVisible(true);
    }

    private void connectListeners(ConflictEditorFrame frame)
    {
        frame.getConnectButton().addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                connectAndFindConflicts();
            }
        });

        frame.getConflictsList().addListSelectionListener(new ListSelectionListener()
        {
            @Override
            public void valueChanged(ListSelectionEvent e)
            {
                selectConflictDocument();
            }
        });

        frame.getDifferencesTable().getSelectionModel().addListSelectionListener(new ListSelectionListener()
        {
            @Override
            public void valueChanged(ListSelectionEvent e)
            {
                selectDifference();
            }
        });
    }

    protected void connectAndFindConflicts()
    {
        try
        {
            String dbUrl = _frame.getDatabaseUrl();
            CouchDb database = new CouchDb(dbUrl);
            ConflictsView conflictsView = new ConflictsView(database);

            ListModel model = new ConflictsListModel(conflictsView.getConflicts());
            _frame.getConflictsList().setModel(model);
        }
        catch (IOException iox)
        {
            PrintStream out = new PrintStream(new ByteArrayOutputStream());
            iox.printStackTrace(out);
            out.close();

            JOptionPane.showMessageDialog(_frame, out.toString());
        }
    }

    protected void selectConflictDocument()
    {
        List<Difference> diffs = new ArrayList<Difference>();
        diffs.add(new Difference("key", "current", "other"));

        TableModel model = new DifferenceTableModel(diffs);
        _frame.getDifferencesTable().setModel(model);
    }

    protected void selectDifference()
    {
        System.out.println("select a difference");
    }
}
